import re

from pyrogram.types import InlineKeyboardButton, InlineKeyboardMarkup


class Button:
    def url(text):
        link_pattern = r"(?:https?://)?(?:www\.)?[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}(?:[/?]\S+)?|tg://\S+$"
        return re.findall(link_pattern, text)

    def text(text):
        button_matches = re.findall(r"\| ([^|]+) - ([^|]+) \|", text)
        text_matches = re.split(r"\| [^|]+ - [^|]+ \|", text)[0].strip() if "|" in text else text.strip()
        return button_matches, text_matches

    def create(text, inline_cmd=None, is_id=None):
        keyboard = []
        button_matches, text_matches = Button.text(text)

        for button_text, button_data in button_matches:
            cb_data, *data_parts = button_data.split(";")

            if not Button.url(cb_data):
                cb_data = f"{inline_cmd} {is_id}_{cb_data}" if inline_cmd and is_id else cb_data

            button = (
                InlineKeyboardButton(button_text, user_id=cb_data)
                if "user" in data_parts
                else (
                    InlineKeyboardButton(button_text, url=cb_data)
                    if Button.url(cb_data)
                    else InlineKeyboardButton(button_text, callback_data=cb_data)
                )
            )

            if "same" in data_parts and keyboard:
                keyboard[-1].append(button)
            else:
                keyboard.append([button])

        return InlineKeyboardMarkup(keyboard), text_matches

    def inline(buttons, row_width=2):
        keyboard = [
            [InlineKeyboardButton(**button_data) for button_data in buttons[i : i + row_width]]
            for i in range(0, len(buttons), row_width)
        ]
        return InlineKeyboardMarkup(keyboard)



from pyrogram import enums


class User:
    @staticmethod
    async def get_user_id(message, username):
        entities = message.entities
        app = message._client

        if entities:
            entity_index = 1 if message.text.startswith("/") else 0
            entity = entities[entity_index]

            if entity.type == enums.MessageEntityType.MENTION:
                return (await app.get_chat(username)).id
            elif entity.type == enums.MessageEntityType.TEXT_MENTION:
                return entity.user.id
        return username

    @staticmethod
    async def user_id(message, text):
        if text.isdigit():
            return int(text)
        else:
            return await User.get_user_id(message, text)

    @staticmethod
    async def ger_rid(message, sender_chat=False):
        text = message.text.strip()
        args = text.split()

        if message.reply_to_message:
            reply = message.reply_to_message
            if reply.from_user:
                user_id = reply.from_user.id
            elif sender_chat and reply.sender_chat and reply.sender_chat.id != message.chat.id:
                user_id = reply.sender_chat.id
            else:
                return None, None

            reason = text.split(None, 1)[1] if len(args) > 1 else None
            return user_id, reason

        if len(args) == 2:
            user = args[1]
            return await User.user_id(message, user), None

        if len(args) > 2:
            user, reason = args[1], " ".join(args[2:])
            return await User.user_id(message, user), reason

        return None, None

    @staticmethod
    async def get_admin(message):
        return [
            member.user.id
            async for member in message._client.get_chat_members(message.chat.id, filter=enums.ChatMembersFilter.ADMINISTRATORS)
        ]

    @staticmethod
    async def get_id(message):
        return (await User.id_and_reason(message))[0]
